@@ -35,15 +35,6 @@
 
 #include "js/GCHashTable.h"
 
-G_BEGIN_DECLS
-
-typedef enum {
-    GJS_BOXED_CREATION_NONE    =  0,
-    GJS_BOXED_CREATION_NO_COPY = (1 << 0)
-} GjsBoxedCreationFlags;
-
-G_END_DECLS
-
 class BoxedPrototype;
 class BoxedInstance;
 
@@ -146,10 +137,13 @@ class BoxedPrototype : public GIWrapperPrototype<BoxedBase, BoxedPrototype,
     static FieldMap* create_field_map(JSContext* cx, GIStructInfo* struct_info);
     GJS_JSAPI_RETURN_CONVENTION
     bool ensure_field_map(JSContext* cx);
+    GJS_JSAPI_RETURN_CONVENTION
+    bool define_boxed_class_fields(JSContext* cx, JS::HandleObject proto);
 
  public:
     GJS_JSAPI_RETURN_CONVENTION
-    bool define_boxed_class_fields(JSContext* cx, JS::HandleObject proto);
+    static bool define_class(JSContext* cx, JS::HandleObject in_object,
+                             GIStructInfo* info);
     GJS_JSAPI_RETURN_CONVENTION
     GIFieldInfo* lookup_field(JSContext* cx, JSString* prop_name);
 };
@@ -203,6 +197,9 @@ class BoxedInstance
                                      GIBaseInfo* interface_info,
                                      JS::HandleValue value);
 
+    GJS_JSAPI_RETURN_CONVENTION
+    static void* copy_ptr(JSContext* cx, GType gtype, void* ptr);
+
     // JS property accessors
 
     GJS_JSAPI_RETURN_CONVENTION
@@ -223,33 +220,22 @@ class BoxedInstance
  public:
     struct NoCopy {};
 
+ private:
     GJS_JSAPI_RETURN_CONVENTION
     bool init_from_c_struct(JSContext* cx, void* gboxed);
     GJS_JSAPI_RETURN_CONVENTION
     bool init_from_c_struct(JSContext* cx, void* gboxed, NoCopy);
+    template <typename... Args>
+    GJS_JSAPI_RETURN_CONVENTION static JSObject* new_for_c_struct_impl(
+        JSContext* cx, GIStructInfo* info, void* gboxed, Args&&... args);
+
+ public:
+    GJS_JSAPI_RETURN_CONVENTION
+    static JSObject* new_for_c_struct(JSContext* cx, GIStructInfo* info,
+                                      void* gboxed);
+    GJS_JSAPI_RETURN_CONVENTION
+    static JSObject* new_for_c_struct(JSContext* cx, GIStructInfo* info,
+                                      void* gboxed, NoCopy);
 };
 
-G_BEGIN_DECLS
-
-GJS_JSAPI_RETURN_CONVENTION
-bool gjs_define_boxed_class(JSContext* cx, JS::HandleObject in_object,
-                            GIStructInfo* info);
-
-GJS_JSAPI_RETURN_CONVENTION
-void*     gjs_c_struct_from_boxed      (JSContext             *context,
-                                        JS::HandleObject       obj);
-GJS_JSAPI_RETURN_CONVENTION
-JSObject* gjs_boxed_from_c_struct      (JSContext             *context,
-                                        GIStructInfo          *info,
-                                        void                  *gboxed,
-                                        GjsBoxedCreationFlags  flags);
-GJS_USE
-bool      gjs_typecheck_boxed          (JSContext             *context,
-                                        JS::HandleObject       obj,
-                                        GIStructInfo          *expected_info,
-                                        GType                  expected_type,
-                                        bool                   throw_error);
-
-G_END_DECLS
-
 #endif  /* __GJS_BOXED_H__ */
